'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const vite = require('vite');
const ejs = require('ejs');
const dotenvExpand = require('dotenv-expand');
const dotenv = require('dotenv');
const pathe = require('pathe');
const fsExtra = require('fs-extra');
const pluginutils = require('@rollup/pluginutils');
const nodeHtmlParser = require('node-html-parser');
const fg = require('fast-glob');
const consola = require('consola');
const colorette = require('colorette');
const htmlMinifierTerser = require('html-minifier-terser');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const dotenv__default = /*#__PURE__*/_interopDefaultLegacy(dotenv);
const fg__default = /*#__PURE__*/_interopDefaultLegacy(fg);
const consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);

function loadEnv(mode, envDir, prefix = "") {
  if (mode === "local") {
    throw new Error(`"local" cannot be used as a mode name because it conflicts with the .local postfix for .env files.`);
  }
  const env = {};
  const envFiles = [
    `.env.${mode}.local`,
    `.env.${mode}`,
    `.env.local`,
    `.env`
  ];
  for (const file of envFiles) {
    const path = lookupFile(envDir, [file], true);
    if (path) {
      const parsed = dotenv__default.parse(fsExtra.readFileSync(path), {
        debug: !!process.env.DEBUG || void 0
      });
      dotenvExpand.expand({
        parsed,
        ignoreProcessEnv: true
      });
      for (const [key, value] of Object.entries(parsed)) {
        if (key.startsWith(prefix) && env[key] === void 0) {
          env[key] = value;
        } else if (key === "NODE_ENV") {
          process.env.VITE_USER_NODE_ENV = value;
        }
      }
    }
  }
  return env;
}
function lookupFile(dir, formats, pathOnly = false) {
  for (const format of formats) {
    const fullPath = pathe.join(dir, format);
    if (fsExtra.existsSync(fullPath) && fsExtra.statSync(fullPath).isFile()) {
      return pathOnly ? fullPath : fsExtra.readFileSync(fullPath, "utf-8");
    }
  }
  const parentDir = pathe.dirname(dir);
  if (parentDir !== dir) {
    return lookupFile(parentDir, formats, pathOnly);
  }
}
function cleanUrl(url) {
  const queryRE = /\?.*$/s;
  const hashRE = /#.*$/s;
  return url.replace(hashRE, "").replace(queryRE, "");
}
async function isDirEmpty(dir) {
  return fsExtra.readdir(dir).then((files) => {
    return files.length === 0;
  });
}

const htmlFilter = pluginutils.createFilter(["**/*.html"]);

const DEFAULT_TEMPLATE = "index.html";
const ignoreDirs = [".", "", "/"];
const bodyInjectRE = /<\/body>/;
function createPlugin(userOptions = {}) {
  const { entry, template = DEFAULT_TEMPLATE, pages = [] } = userOptions;
  let viteConfig;
  let env = {};
  return {
    name: "vite:html",
    configResolved(resolvedConfig) {
      viteConfig = resolvedConfig;
      env = loadEnv(viteConfig.mode, viteConfig.root, "");
    },
    config(conf) {
      return vite.mergeConfig(conf, {
        build: {
          rollupOptions: {
            input: createInput(userOptions, conf)
          }
        }
      });
    },
    configureServer(server) {
      server.middlewares.use(async (req, res, next) => {
        const url = cleanUrl(req.url || "");
        if (!htmlFilter(url) && url !== "/") {
          return next();
        }
        try {
          const htmlName = url === "/" ? DEFAULT_TEMPLATE : url.replace("/", "");
          const page = getPage(userOptions, htmlName, viteConfig);
          const { injectOptions = {} } = page;
          let html = await getHtmlInPages(page, viteConfig.root);
          html = await renderHtml(html, {
            injectOptions,
            viteConfig,
            env,
            entry: page.entry || entry
          });
          html = await server.transformIndexHtml?.(url, html, req.originalUrl);
          res.end(html);
        } catch (e) {
          consola__default.log(e);
        }
      });
    },
    transform(code, id) {
      if (viteConfig.command === "build" && htmlFilter(id)) {
        const htmlName = id.match("[^/]+(?!.*/)")?.[0] ?? DEFAULT_TEMPLATE;
        const page = getPage(userOptions, htmlName, viteConfig);
        const { injectOptions = {} } = page;
        return getHtmlInPages(page, viteConfig.root).then((html) => {
          return renderHtml(html, {
            injectOptions,
            viteConfig,
            env,
            entry: page.entry || entry
          }).then((resultHtml) => {
            return {
              code: resultHtml,
              map: null
            };
          });
        });
      }
      return {
        code,
        map: null
      };
    },
    async closeBundle() {
      const outputDirs = [];
      if (isMpa(viteConfig) || pages.length) {
        for (const page of pages) {
          const dir = pathe.dirname(page.template);
          if (!ignoreDirs.includes(dir)) {
            outputDirs.push(dir);
          }
        }
      } else {
        const dir = pathe.dirname(template);
        if (!ignoreDirs.includes(dir)) {
          outputDirs.push(dir);
        }
      }
      const cwd = pathe.resolve(viteConfig.root, viteConfig.build.outDir);
      const htmlFiles = await fg__default(outputDirs.map((dir) => `${dir}/*.html`), { cwd: pathe.resolve(cwd), absolute: true });
      await Promise.all(htmlFiles.map((file) => fsExtra.move(file, pathe.resolve(cwd, pathe.basename(file)), { overwrite: true })));
      const htmlDirs = await fg__default(outputDirs.map((dir) => dir), { cwd: pathe.resolve(cwd), onlyDirectories: true, absolute: true });
      await Promise.all(htmlDirs.map(async (item) => {
        const isEmpty = await isDirEmpty(item);
        if (isEmpty) {
          return fsExtra.remove(item);
        }
      }));
    }
  };
}
function createInput({ pages = [], template = "" }, viteConfig) {
  const input = {};
  if (isMpa(viteConfig) || pages?.length) {
    const templates = pages.map((page) => page.template);
    templates.forEach((temp) => {
      const file = pathe.basename(temp);
      const key = file.replace(/\.html/, "");
      input[key] = pathe.resolve(viteConfig.root, temp);
    });
    return input;
  } else {
    const dir = pathe.dirname(template);
    if (ignoreDirs.includes(dir)) {
      return void 0;
    } else {
      const file = pathe.basename(template);
      const key = file.replace(/\.html/, "");
      return {
        [key]: pathe.resolve(viteConfig.root, template)
      };
    }
  }
}
async function renderHtml(html, config) {
  const { injectOptions, viteConfig, env, entry } = config;
  const { data, ejsOptions } = injectOptions;
  const ejsData = {
    ...viteConfig?.env ?? {},
    ...viteConfig?.define ?? {},
    ...env || {},
    ...data
  };
  let result = await ejs.render(html, ejsData, ejsOptions);
  if (entry) {
    result = removeEntryScript(result);
    result = result.replace(bodyInjectRE, `<script type="module" src="${vite.normalizePath(`/${entry}`)}"><\/script>
</body>`);
  }
  return result;
}
function getPage({ pages = [], entry, template = "", inject = {} }, name, viteConfig) {
  let page;
  if (isMpa(viteConfig) || pages?.length) {
    page = getPageConfig(name, pages, DEFAULT_TEMPLATE);
  } else {
    page = createSpaPage(entry, template, inject);
  }
  return page;
}
function isMpa(viteConfig) {
  const input = viteConfig?.build?.rollupOptions?.input ?? void 0;
  return typeof input !== "string" && Object.keys(input || {}).length > 1;
}
function removeEntryScript(html) {
  if (!html) {
    return html;
  }
  const root = nodeHtmlParser.parse(html);
  const scriptNodes = root.querySelectorAll("script[type=module]") || [];
  const removedNode = [];
  scriptNodes.forEach((item) => {
    removedNode.push(item.toString());
    item.parentNode.removeChild(item);
  });
  removedNode.length && consola__default.warn(`vite-plugin-html: Since you have already configured entry, ${colorette.dim(removedNode.toString())} is deleted. You may also delete it from the index.html.
        `);
  return root.toString();
}
function createSpaPage(entry, template, inject = {}) {
  return {
    entry,
    filename: "index.html",
    template,
    injectOptions: inject
  };
}
function getPageConfig(htmlName, pages, defaultPage) {
  const defaultPageOption = {
    filename: defaultPage,
    template: `./${defaultPage}`
  };
  const page = pages.filter((page2) => page2.filename === htmlName)?.[0];
  return page ?? defaultPageOption ?? void 0;
}
function getHtmlInPages(page, root) {
  const htmlPath = getHtmlPath(page, root);
  return readHtml(htmlPath);
}
function getHtmlPath(page, root) {
  const { template } = page;
  const templatePath = template.startsWith(".") ? template : `./${template}`;
  return pathe.resolve(root, templatePath);
}
async function readHtml(path) {
  if (!fsExtra.existsSync(path)) {
    throw new Error(`html is not exist in ${path}`);
  }
  return await fsExtra.readFile(path).then((buffer) => buffer.toString());
}

function getOptions(minify) {
  return {
    collapseWhitespace: minify,
    keepClosingSlash: minify,
    removeComments: minify,
    removeRedundantAttributes: minify,
    removeScriptTypeAttributes: minify,
    removeStyleLinkTypeAttributes: minify,
    useShortDoctype: minify,
    minifyCSS: minify
  };
}
async function minifyHtml(html, minify) {
  if (typeof minify === "boolean" && !minify) {
    return html;
  }
  let minifyOptions = minify;
  if (typeof minify === "boolean" && minify) {
    minifyOptions = getOptions(minify);
  }
  return await htmlMinifierTerser.minify(html, minifyOptions);
}
function createMinifyHtmlPlugin({
  minify = true
} = {}) {
  return {
    name: "vite:minify-html",
    enforce: "post",
    async generateBundle(_, outBundle) {
      if (minify) {
        for (const bundle of Object.values(outBundle)) {
          if (bundle.type === "asset" && htmlFilter(bundle.fileName) && typeof bundle.source === "string") {
            bundle.source = await minifyHtml(bundle.source, minify);
          }
        }
      }
    }
  };
}

consola__default.wrapConsole();
function createHtmlPlugin(userOptions = {}) {
  return [createPlugin(userOptions), createMinifyHtmlPlugin(userOptions)];
}

exports.createHtmlPlugin = createHtmlPlugin;
